# Copyright (c) 2012 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from PyQt4.QtGui import QFileDialog, QLayout

from ....model import adapt
from ....ui import IFilesystemLocationEditor

from ..editor_layout_adapter import EditorLayoutAdapter

from .location_editor_components import LocationEditorComponents


@adapt(QLayout, to=IFilesystemLocationEditor)
class QLayoutIFilesystemLocationEditorAdapter(EditorLayoutAdapter, LocationEditorComponents):
    """ An adapter to implement IFilesystemLocationEditor for a filesystem
    location editor.
    """

    @classmethod
    def isadaptable(cls, adaptee):
        """ Determine if this adapter can adapt a particular layout.

        :param adaptee:
            is the layout.
        :return:
            ``True`` if the layout can be adapted.
        """

        return cls.usable_editor(adaptee)

    def configure(self, properties):
        """ Configure the editor. """

        line_edit = self.line_edit

        # For automation purposes give the line edit an the id if it doesn't
        # already have one.
        if line_edit.objectName() == '':
            line_edit.setObjectName(self.id)

        # Connect things up.
        line_edit.textEdited.connect(self.qt4_notify)
        self.browse_button.clicked.connect(self._on_clicked)

        # Apply any properties to the line edit.
        try:
            line_edit.pyqtConfigure(**properties)
        except AttributeError:
            pass

    @IFilesystemLocationEditor.read_only.getter
    def read_only(self):
        """ Invoked to get the read-only state. """

        return self.line_edit.isReadOnly()

    @read_only.setter
    def read_only(self, value):
        """ Invoked to set the read-only state. """

        self.line_edit.setReadOnly(value)
        self.browse_button.setVisible(not value)

    @IFilesystemLocationEditor.value.getter
    def value(self):
        """ Invoked to get the editor's value. """

        return self.line_edit.text()

    @value.setter
    def value(self, value):
        """ Invoked to set the editor's value. """

        # Save for the notifier.
        self.qt4_old_value = value

        self.line_edit.setText(value)

    def set_focus(self):
        """ Give the focus to the view. """

        self.line_edit.setFocus()

    def _on_clicked(self):
        """ Invoked when the browse button is clicked. """

        value = self.value
        mode = self.mode
        filter = self.filter
        line_edit = self.line_edit

        if mode == 'open_file':
            value = QFileDialog.getOpenFileName(line_edit, directory=value,
                    filter=filter)
            ok = (value != '')

        elif mode == 'save_file':
            value = QFileDialog.getSaveFileName(line_edit, directory=value,
                    filter=filter)
            ok = (value != '')

        elif mode == 'directory':
            value = QFileDialog.getExistingDirectory(line_edit,
                    directory=value)
            ok = (value != '')

        line_edit.activateWindow()
        line_edit.setFocus()

        if ok:
            self.value = value
