# Copyright (c) 2012 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from PyQt4.QtCore import QPyNullVariant, QSettings

from ....model import adapt, Adapter
from ....settings import ISettingsStorage


@adapt(QSettings, to=ISettingsStorage)
class QSettingsISettingsStorageAdapter(Adapter):
    """ An adapter to implement ISettingsStorage for a QSettings. """

    def begin_group(self, group):
        """ Begin a group.

        :param group:
            is the name of the group.
        """

        self.adaptee.beginGroup(group)

    def end_group(self):
        """ End the current group. """

        self.adaptee.endGroup()

    def flush(self):
        """ Ensure that all settings changes are written to persistent storage.
        """

        self.adaptee.sync()

    def read_value(self, name):
        """ Read the value of a setting.

        :param name:
            is the name of the setting.
        :return:
            the value of the setting, or None if there is no such setting.
        """

        value = self.adaptee.value(name)

        if isinstance(value, QPyNullVariant):
            value = None

        return value

    def write_value(self, name, value):
        """ Write the value of a setting.

        :param name:
            is the name of the setting.
        :param value:
            is the value of the setting.  If this is None then the setting is
            removed.
        """

        if value is None:
            self.adaptee.remove(name)
        else:
            self.adaptee.setValue(name, value)
