# Copyright (c) 2012 Riverbank Computing Limited.
#
# This file is part of dip.
#
# This file may be used under the terms of the GNU General Public License
# v2 or v3 as published by the Free Software Foundation which can be found in
# the files LICENSE-GPL2.txt and LICENSE-GPL3.txt included in this package.
# In addition, as a special exception, Riverbank gives you certain additional
# rights.  These rights are described in the Riverbank GPL Exception, which
# can be found in the file GPL-Exception.txt in this package.
#
# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.


from ...model import Adapter
from ...ui import IView


class ObjectAdapter(Adapter):
    """ The ObjectAdapter class is a base class for adapters than implement the
    :class:`~dip.ui.IObject` interface.
    """

    @IView.id.getter
    def id(self):
        """ Invoked to get the id. """

        return self.adaptee.objectName()
    
    @id.setter
    def id(self, value):
        """ Invoked to set the id. """
        
        self.adaptee.setObjectName(value)
        
    def configure(self, properties):
        """ Configure the object using a mapping of toolkit specific property
        names and values.

        :param properties:
            is the mapping of the properties.
        """

        obj = self.adaptee

        blocked = obj.blockSignals(True)

        try:
            obj.pyqtConfigure(**properties)
        except AttributeError:
            pass

        obj.blockSignals(blocked)
